(************************************************************
Author: Deepak Shenoy
        shenoy@agnisoft.com
Copyright (C) 2000 Agni Software Pvt. Ltd.
All Rights Reserved.
http://www.agnisoft.com

This form displays various types of binding in ADSI.
1. Getting provider names using the built in ADSI functions
2. Using my helper functions for enumeration
3. Using ADsOpenObject for binding
4. Setting object information
5. Setting multivalued object info (requires Windows 2000 and Active Directory)

REMEMBER:
There are some constants defined that have to be changed so this form will actually
work on your system.
********************************************************)
unit Mainform;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ActiveX, OleCtrls, comobj, ActiveDs_TLB, Db, ADODB, variants;

type
  TForm1 = class(TForm)
    ResultList: TListBox;
    Button1: TButton;
    Button3: TButton;
    Button2: TButton;
    Button4: TButton;
    Button5: TButton;
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
    procedure CallBackProc( Obj : IADs);
  end;

var
  Form1: TForm1;

implementation

uses adshlp;

{$R *.DFM}

// CONSTANTS
// To be changed: will almost definitely not work on your system unless changed.
const DOMAINNAME = 'AGNI';     // WinNT/2000 domain
      USERNAME   = 'deepak';   // User name 
      TESTUSER   = 'arun';     // Username for use in AdsOpenObject
      TESTPWD    = 'arun';      // password for use in AdsOpenObject
      LDAPSTRING = 'CN=deepak,CN=Users,DC=AGNISOFT,DC=COM';
      
      
procedure TForm1.Button1Click(Sender: TObject);
var NSContainer : IADsContainer;
    Enum : IEnumVariant;
    hr : integer;
    varArr : OleVariant;
    lNumElements : ULONG;
    item : IADs;
    s : String;
begin
  ResultList.Items.Clear;
  NSContainer:=nil;
  ADsGetObject( 'ADs:', IADsContainer, NSContainer);
  Enum :=nil;
  hr := ADsBuildEnumerator(NSContainer,Enum);
  while SUCCEEDED(hr) do
  begin
    hr := ADsEnumerateNext(Enum,1,
                    varArr,lNumElements);

    if (lNumElements<=0) then
      break;

    IDispatch(varArr).QueryInterface(IADs, item) ;
    s := item.ADsPath;
    ResultList.Items.Add(s);
 end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  CoInitialize(nil);
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  CoUninitialize;
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
   ADsEnumerateObjects('ADs:', CallBackProc);
end;

procedure TForm1.CallBackProc(Obj: IADs);
var
    s : string;
begin
  s := Obj.AdsPath;
  ResultList.Items.Add(s);
end;

procedure TForm1.Button3Click(Sender: TObject);
var hr : integer;
    obj : IADs;
begin
  hr := ADsOpenObject('WinNT://' + DOMAINNAME, TESTUSER,TESTPWD,
           ADS_PROMPT_CREDENTIALS , IADs, obj );
  if Succeeded(hr) then
     ShowMessage(obj.Name + ',' + obj.ADsPath);
end;

procedure TForm1.Button4Click(Sender: TObject);
var obj : Variant;
begin
  obj := ADsHlp.GetObject('WinNT://' + DOMAINNAME + '/' + USERNAME);
  obj.Put( 'FullName', 'Deepak');
  obj.SetInfo;
  obj := NULL;
end;

procedure TForm1.Button5Click(Sender: TObject);
var obj : IAds;
begin
  ADsGetObject('LDAP://' + LDAPSTRING, IAds, Obj);
  obj.PutEx(ADS_PROPERTY_APPEND, 'otherHomePhone', VarArrayOf(['333-3333']) );
  obj.SetInfo;

  obj.PutEx(ADS_PROPERTY_DELETE, 'otherHomePhone', VarArrayOf(['111-1111', '222-2222']));
  obj.SetInfo;
  obj.PutEx(ADS_PROPERTY_UPDATE, 'otherHomePhone', VarArrayOf(['888-8888','999-9999']));
  obj.SetInfo;
  obj.PutEx(ADS_PROPERTY_CLEAR, 'otherHomePhone',  NULL);
  obj.SetInfo;
end;

end.
