(************************************************************
Author: Deepak Shenoy
        shenoy@agnisoft.com
Copyright (C) 2000 Agni Software Pvt. Ltd.
All Rights Reserved.
http://www.agnisoft.com

Description:
ADSI Namespace provider for BDE
You must execute BDEADSI.REG, and then register this server for it
to work. You can check out this by typing BDE://DBDEMOS in ADSvw.exe
(provided with the platform SDK)
Note: Schemas and Property Cache are not implemented
********************************************************)
unit AdsBdeNS;

interface

uses
  ComObj, ActiveX, BDEProv_TLB, StdVcl,ActiveDS_TLB, ADsCore, AdsDebug, Windows, dbTables, db,
  classes;

type
  { Supports : IParseDisplayName,
               IADS
               IADSContainer
               IADsOpenDSObject}
  TBDENameSpace = class(TAdsAutoObject, IBDENameSpace, IParseDisplayName, IADsOpenDsObject, IADs, IADsContainer)
  protected
    { Protected declarations }
    FHints,FFilter : OleVariant;
    FAliases : TStringList;

    {IParseDisplayName}
    function ParseDisplayName(const bc: IBindCtx; pszDisplayName: POleStr;
      out chEaten: Longint; out mkOut: IMoniker): HResult; stdcall;
    {IAdsOpenDsObject}
    function OpenDSObject(const lpszDNName: WideString; const lpszUserName: WideString;
                  const lpszPassword: WideString; lnReserved: Integer): IDispatch; safecall;
    function  Get_ADsPath: WideString; safecall;
    function Get(const bstrName: WideString): OleVariant; safecall;
    function Get_Class_: WideString; safecall;
    function Get_GUID: WideString; safecall;
    function Get_Name: WideString; safecall;
    function Get_Parent: WideString; safecall;
    function Get_Schema: WideString; safecall;
    function GetEx(const bstrName: WideString): OleVariant; safecall;
    procedure GetInfo; safecall;
    procedure GetInfoEx(vProperties: OleVariant; lnReserved: Integer);
      safecall;
    procedure Put(const bstrName: WideString; vProp: OleVariant); safecall;
    procedure PutEx(lnControlCode: Integer; const bstrName: WideString;
      vProp: OleVariant); safecall;
    procedure SetInfo; safecall;

    procedure Initialize; override;
    destructor Destroy; override;
    {IADsContainer}
    function GetObject(const ClassName, RelativeName: WideString): IDispatch;
                        safecall;
    function Create(const ClassName, RelativeName: WideString): IDispatch;
      safecall;
    procedure Delete(const bstrClassName, bstrRelativeName: WideString);
      safecall;
    function CopyHere(const SourceName, NewName: WideString): IDispatch;
      safecall;
    function MoveHere(const SourceName, NewName: WideString): IDispatch;
      safecall;
    function Get__NewEnum: IUnknown; safecall;
    function Get_Filter: OleVariant; safecall;
    procedure Set_Filter(Value: OleVariant); safecall;
    function Get_Count: Integer; safecall;
    function Get_Hints: OleVariant; safecall;
    procedure Set_Hints(Value: OleVariant); safecall;
  end;

implementation

uses ComServ, Sysutils, enumns, genobject;

{ TBDENameSpace }

function TBDENameSpace.Get_ADsPath: WideString;
begin
  Result := 'BDE:';
end;

function TBDENameSpace.OpenDSObject(const lpszDNName, lpszUserName,
  lpszPassword: WideString; lnReserved: Integer): IDispatch;
begin
  //Result := NULL;
  NotImpl;
end;

function TBDENameSpace.ParseDisplayName(const bc: IBindCtx;
  pszDisplayName: POleStr; out chEaten: Integer;
  out mkOut: IMoniker): HResult;
var szAlias, szName : string;
    unk : IUnknown;
    obj : TBDEGenObject;

begin
  szName := OleStrToString( pszDisplayName );

  // Check if we've got the correct namespace
  if Uppercase(Copy(szName,1, 4)) <> 'BDE:' then
  begin
     Result := E_FAIL;
     Exit;
  end;
  chEaten := 0;
  // parse the name, get the things required
  if (szName = 'BDE:') or (szName='BDE://') then
  begin
    unk := Self; // namespace itself
    _AddRef;
    Result := CreatePointerMoniker(unk,mkOUt);
    Inc(chEaten, Length(Widestring(szName)));
    Result := S_OK;
  end
  else
  begin
    szAlias := Copy(szName, 7, LEngth(szName));
    if FAliases.IndexOf( szAlias ) <> -1 then
    begin
      // create an object with the BDE alias 
      obj := TBDEGenObject.Create;
      obj.Alias := szAlias;
      unk := obj;
      _AddRef;
      Result := CreatePointerMoniker(unk,mkOUt);
      Inc(chEaten, Length(Widestring(szName)));
    end
    else
      result := E_FAIL;
  end;
end;

function TBDENameSpace.Get(const bstrName: WideString): OleVariant;
begin
  Result := inherited Get(bstrName);
end;

function TBDENameSpace.Get_Class_: WideString;
begin
  Result := inherited Get_Class_;
end;

function TBDENameSpace.Get_GUID: WideString;
begin
  Result := GUIDToString( IBDENameSpace);
end;

function TBDENameSpace.Get_Name: WideString;
begin
   Result := 'BDENameSpace';
end;

function TBDENameSpace.Get_Parent: WideString;
begin
  Result := inherited Get_Parent;
end;

function TBDENameSpace.Get_Schema: WideString;
begin
  //Result := inherited Get_Schema;
  raise EOleSysError.Create('Not Impl', E_NOTIMPL, 0);
end;

function TBDENameSpace.GetEx(const bstrName: WideString): OleVariant;
begin
  Result := inherited GetEx(bstrName);
end;

procedure TBDENameSpace.GetInfo;
begin
  inherited GetInfo;
end;

procedure TBDENameSpace.GetInfoEx(vProperties: OleVariant;
  lnReserved: Integer);
begin
  inherited GetInfoEx(vProperties, lnReserved);
end;

procedure TBDENameSpace.Put(const bstrName: WideString; vProp: OleVariant);
begin
  inherited Put(bstrName, vProp);
end;

procedure TBDENameSpace.PutEx(lnControlCode: Integer;
  const bstrName: WideString; vProp: OleVariant);
begin
  inherited PutEx(lnControlCode, bstrName, vProp);
end;

procedure TBDENameSpace.SetInfo;
begin
    raise EOleSysError.Create('Not Impl', E_NOTIMPL, 0);
end;

procedure TBDENameSpace.Initialize;
begin
  inherited;
  _Parent := 'ADs:';
  _Name := 'BDE:';
  FAliases := TStringList.Create;
  Session.GetAliasNames(FAliases);
end;

function TBDENameSpace.GetObject(const ClassName,
  RelativeName: WideString): IDispatch;
begin
  ASendDebug( ClassName + ':' + RelativeName );
end;

function TBDENameSpace.Create(const ClassName,
  RelativeName: WideString): IDispatch;
begin
  NotImpl;
end;

procedure TBDENameSpace.Delete(const bstrClassName,
  bstrRelativeName: WideString);
begin
  NotImpl;
end;

function TBDENameSpace.CopyHere(const SourceName,
  NewName: WideString): IDispatch;
begin
  NotImpl;
end;

function TBDENameSpace.MoveHere(const SourceName,
  NewName: WideString): IDispatch;
begin
  NotImpl;
end;

function TBDENameSpace.Get__NewEnum: IUnknown;
begin
  Result := TEnumBDENamespace.Create(FAliases);
  Result._AddRef;
end;

function TBDENameSpace.Get_Filter: OleVariant;
begin
  Result := FFilter;
end;

procedure TBDENameSpace.Set_Filter(Value: OleVariant);
begin
  FFilter := Value;
end;

function TBDENameSpace.Get_Count: Integer;
begin
  Result := FAliases.Count;
end;

function TBDENameSpace.Get_Hints: OleVariant;
begin
  Result := FHints;
end;

procedure TBDENameSpace.Set_Hints(Value: OleVariant);
begin
  FHints := Value;
end;

destructor TBDENameSpace.Destroy;
begin
  inherited;
  FALiases.Free;
end;

initialization
  TAutoObjectFactory.Create(ComServer, TBDENameSpace, Class_BDENameSpace,
    ciMultiInstance, tmBoth);
end.

